
/******************************** INCLUDES ************************************/
#include "main.h"
#include "stdbool.h"
#include "stdint.h"
#include "string.h"

/*---------------------------- END OF INCLUDES -------------------------------*/





/******************************** DEFINES *************************************/

// Display Port and Pins definition
#define display_Port GPIOA		// GPIO Port being used
#define data_Pin GPIO_PIN_8		// Data pin
#define cs_Pin GPIO_PIN_9		// Chip Select pin
#define clock_Pin GPIO_PIN_10	// Clock pin

// Number of 8x8 Matrix displays being used
#define num_displays 4

// Buffer size of data sent from Bluetooth
#define buffer_size 20

/*---------------------------- END OF DEFINES --------------------------------*/





/**************************** FUNCTION PROTOTYPES *****************************/


/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Delay_ms
	Description:  Creates a time gap in the program flow

	Parameters (1) :
  		time = Time in milliseconds

	Return value:  None
	Function call example:  Delay_ms(200); - creates a 200 millisecond time delay

*`````````````````````````````````````````````````````````````````````````````*/
void Delay_ms (uint16_t time);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Send_byte
	Description:  Sends a byte to the MAX7219 display driver.
				  Used by Send_data() & Set_LED() functions.

	Parameters (1) :
  		byte = Byte to be sent

	Return value:  None
	Function call example:  Send_byte(0xF6);

*`````````````````````````````````````````````````````````````````````````````*/
void Send_byte (uint8_t byte);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Send_data
	Description:  Sends data to a specific address of MAX7219 display driver.
				  Used by Display_init() & Set_brightness() functions.

	Parameters (2) :
		address = Address byte to which data needs to be sent
  		data = Data byte to be sent to the address

	Return value:  None
	Function call example:  Send_data(0xF6, 0x3A);

*`````````````````````````````````````````````````````````````````````````````*/
void Send_data (uint8_t address, uint8_t data);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Set_LED
	Description:  Turns ON / OFF a specific LED of the display.
				  Used by Set_col() function.

	Parameters (3) :
  		row = row value of the LED ---> input values: 1 to 8
  		col = column value of the LED ---> input values: 1 to 32
		value = 1 for ON or 0 for OFF

	Return value:  None
	Function call example:  Set_LED(8, 23, 1); - will make LED at row 8, column 23 glow

*`````````````````````````````````````````````````````````````````````````````*/
void Set_LED (uint8_t row, uint8_t col, uint8_t value);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Set_col
	Description:  Sets an entire column of LEDs of the display
				  Used by several subsequent functions.

	Parameters (2) :
  		col = column of the display selected ---> input values: 1 to 32
		pattern = 8-bit value where 1's represent ON LEDs and 0's represent OFF LEDs

	Return value:  None
	Function call example:  Set_col(31, 0b10101010); - will make alternate LEDs of column 31 glow

*``````````````````````````````````````````````````````````````````````````````*/
void Set_col(uint8_t col, uint8_t pattern);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Clear_display
	Description:  Clears the display
				  Used by Display_init() function

	Parameters : None

	Return value:  None
	Function call example:  Clear_display();

*`````````````````````````````````````````````````````````````````````````````*/
void Clear_display(void);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Display_init
	Description:  Initializes the display

	Parameters (1) :
		brightness = brightness of display ---> input values: 0-15 (15 is max brightness)

	Return value:  None
	Function call example:  Display_init();

*`````````````````````````````````````````````````````````````````````````````*/
void Display_init (uint8_t brightness);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Set_brightness
	Description:  Sets the brightness of the display.
				  Used by the Scroll_message() function

	Parameters (1) :
		brightness = brightness of display ---> input values: 0-15 (15 is max brightness)

	Return value:  None
	Function call example:  Set_brightness(5);

*`````````````````````````````````````````````````````````````````````````````*/
void Set_brightness(uint8_t brightness);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Write_letter
	Description:  Write a character to a specific 8x8 LED matrix of the display

	Parameters (2) :
		letter = Letter to be written ---> input values: 'A' to 'Z' & blank space (' ')
		disp_num = 8x8 display number to be written to ----> input values: 1-4 (if you have 4 8x8 displays)

	Return value:  None
	Function call example:  Write_letter('S', 4); - writes 'S' to 4th display

*`````````````````````````````````````````````````````````````````````````````*/
void Write_letter (char letter,  uint8_t disp_num);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Shift_left
	Description:  Shifts the display LEFT
				  Used by Shift_letter() function

	Parameters : None

	Return value:  None
	Function call example:  Shift_left();

*`````````````````````````````````````````````````````````````````````````````*/
void Shift_left(void);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Shift_right
	Description:  Shifts the display RIGHT
				  Used by Shift_letter() function

	Parameters : None

	Return value:  None
	Function call example:  Shift_right();

*`````````````````````````````````````````````````````````````````````````````*/
void Shift_right(void);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Shift_letter
	Description:  Shifts a single character on the display in a specified speed
				  and direction.
				  Used by Scroll_message() function

	Parameters (3) :
		letter = Letter to be written ---> input values: 'A' to 'Z' & blank space (' ')
		direction = direction of scroll ---> input values: 'L' for left, 'R' for right
		speed = scrolling speed ---> input values: 0-10 (10 is max speed)

	Return value:  None
	Function call example:  Shift_letter('M', 'L', 7);

*``````````````````````````````````````````````````````````````````````````````*/
void Shift_letter (char letter, char direction, uint32_t speed);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Scroll_message
	Description:  Scrolls a message on the display with a specified speed,
				  direction and brightness

	Parameters (4) :
		*string = pointer to the message string ---> input values - either string
				  literal or pointer to string variable
		direction = scroll direction ---> input values: 'L' for left, 'R' for right
		speed = scrolling speed ---> input values: 'L','M','H' for Low, Medium & High
		brightness = display brightness ---> input values: 'L','M','H' for Low, Medium & High

	Return value:  None
	Function call example:  Scroll_message('WELCOME', 'L', 'H', 'M');
							- Scrolls "WELCOME" in LEFT direction at HIGH speed and MEDIUM brightness

*``````````````````````````````````````````````````````````````````````````````*/
void Scroll_message (char *string, char direction, char speed, char brightness);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Extract_values
	Description:  Extracts the message, direction, speed and brightness values
				  from the data buffer received from Bluetooth.
				  Used by Read_bluetooth() function

	Parameters : None

	Return value:  None
	Function call example:  Extract_values();

*``````````````````````````````````````````````````````````````````````````````*/
void Extract_values (void);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Remove_escape_sequence
	Description:  Removes escape sequences at the end of the Bluetooth data buffer
				  Used by Read_bluetooth() function

	Parameters (3) :
		*pSource = Pointer to data buffer from which escape sequences are to be removed
		*pDestination = Pointer to data buffer in which cleaned buffer data is to be stored
		size = Size of source data buffer (pSource)

	Return value:  None
	Function call example:  Remove_escape_sequence(raw_message, clean_message, 20);

*``````````````````````````````````````````````````````````````````````````````*/
void Remove_escape_sequence(char *pSource ,char *pDestination, uint8_t size);




/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Read_bluetooth
	Description:  Receives data from Bluetooth and extracts the message, direction,
				  speed and brightness values.

	Parameters : None

	Return value:  None
	Function call example:  Read_bluetooth();

*``````````````````````````````````````````````````````````````````````````````*/
void Read_bluetooth(void);




/*------------------------ END OF FUNCTION PROTOTYPES -------------------------*/



